

#if !defined __VGEV_DRAW_H
#define __VGEV_DRAW_H


#ifdef WIN32
#include "WIN_W32.h"

typedef struct {

   HWND window;
   UINT type;
   WPARAM wparam;
   LPARAM lparam;
} event_message_type;

#else 
#include "WIN_X.h"
#endif


#include "vissim.h"

#include "vgev_tile.h"
#include "vgev_mouse.h"
#include "vgev_map.h"


#define FLAG_NULL      0x00
#define FLAG_UPDATE    0x01
#define FLAG_GREYSCALE 0x02
#define FLAG_FOLLOW    0x04
#define FLAG_DONE      0x08


/* **************************************************************
************************************************************** */
class mode_status {

   public:
      double xcenter, ycenter;
      double xoffset, yoffset;
      double master_scale;
      int initflag;

      mode_status() { initflag = 0; }
};


/* **************************************************************
************************************************************** */
class canvas : public dbl_llist {

   protected:
      vector2d circle[32];
      double scenario_radius;

      double scale;
      double xoffset, yoffset;		// center of screen - world coords
      double xcenter, ycenter;		// center of screen - screen coords

      int mode_count;
      mode_status *mode_stat;

      tile_manager_type tileman;

      void update_display_background();

   public:

#ifdef WIN32
      WOGLwindow win;
#else
      XOGLwindow win;
//      Cursor cursor;
#endif

      int redraw_flag;

      mouse_handler_type *mouse_mode;
      string_type readout;
      
      unsigned int flags;

      dbl_llist_manager primitive_manager;
      WVSmap earth_map;

      canvas(int mc);
      canvas(int mc, mouse_handler_type *m);
      
      virtual ~canvas() { if (mouse_mode) delete mouse_mode; }

      void tpColor3ub(unsigned char r, unsigned char g, unsigned char b, int gflag);
      void tpColor3ubv(unsigned char *v, int gflag);
      void tpColor4ubv(unsigned char *v, int gflag);

      void init(int argc, char **argv);
      void resize(int w, int h);
      void recenter(double *x, double *y);
      void xy2lonlat(int x, int y, double *lon, double *lat);
      int clip(double lon, double lat, int *xout, int *yout);

      void recenter(double *radius);
      void set_scale(double *s);
      double get_scale() { return scale; }
      
      void DrawString(int x, int y, char *string, int length);

      virtual void update_display();
      virtual int process_input(iface *isim, void *event);
};


extern dbl_llist_manager canvas_manager;


#endif
