

#include <string.h>

#include "global.h"
#include "pstring.h"
#include "dataio.h"
#include "user_io.h"

#include "vissim.h"


/* **************************************************************
  This class is the base class for any GUI' controller for the system
************************************************************** */
class bsdata : public binary_data {

   public:
      int compare(binary_data *x);

      vector4uc color;
      string_type key;

      virtual ~bsdata() {}
};

string_type terrain_name;

double camera_dist = 1;		// meters
double camera_height = 0;	// meters
double camera_azimuth = 0;	// radians
double camera_angular_velocity = deg2rad(5.0); // radians

symbol_lut_type symbol_lut;
binary_tree colortree;
unsigned int security_level;
int controller = CONTROLMOUSE;

int local_game_fogflag[3] = {0,0,0};
double local_game_fog_start = 0;
double local_game_fog_end = 0;
vector4uc local_game_fog_color = {255,255,255,0};

int openglflag = 11;

string_type overlay_name;
int overlay_intensity = 255;

string_type map_indexname;
vector4uc tile_color = { 255, 255, 255, 255 };
vector3d controller_speed = { 1024.0, 1024.0, 1024.0};

int scramnet_flag = 0;

double backplane = -999999.0;

void default_setup_CB(FILE *infile, char *token) {}
void default_setup_out_CB(FILE *outfile) {}

void *setup_CB     = (void *)default_setup_CB;
void *setup_out_CB = (void *)default_setup_out_CB;

string_type PIXCON_PATH;
string_type DATANAME_SEMOBJ("config/semobj_data");
string_type DATANAME_SETUP("config/setup.ini");


/* ****************************************************************************
**************************************************************************** */
int bsdata::compare(binary_data *x) {

   return key.stringcmp(&((bsdata *)x)->key);
}

      
/* ****************************************************************************
**************************************************************************** */
dbl_llist *symbol_lut_type::find(int code) {

   symbol_correlation_type *str;

   for (str=(symbol_correlation_type *)symbol_lut.head; str && str->sym_id != code; str=(symbol_correlation_type *)str->next);
   return str;
}


/* ****************************************************************************
**************************************************************************** */
dbl_llist *symbol_lut_type::find(char *code) {

   symbol_correlation_type *str;

   for (str=(symbol_correlation_type *)symbol_lut.head; str && str->sym_sim.stringcmp(code); str=(symbol_correlation_type *)str->next);
   return str;
}


/* ****************************************************************************
**************************************************************************** */
void symbol_lut_type::dump(FILE *outfile) {

   symbol_correlation_type *ptr;

   for (ptr = (symbol_correlation_type *)head; ptr; ptr = (symbol_correlation_type *)ptr->next) {
      fprintf(outfile, "\t%d {\n", ptr->sym_id);
      fprintf(outfile, "\t\tlabel %s\n", ptr->sym_label.string);
      fprintf(outfile, "\t\tdis_code %s\n", ptr->sym_sim.string);
      fprintf(outfile, "\t\tjpix_code %d\n", ptr->sym_3D);
      fprintf(outfile, "\t\tburst_code %d %d %d %d\n", ptr->burst_warhead, ptr->burst_fuze, ptr->burst_quantity, ptr->burst_rate);
      fprintf(outfile, "\t\tgev_code %d\n", ptr->sym_2D);
      fprintf(outfile, "\t\tgev_color %d\n", ptr->sym_2D_colorname.string);
      fprintf(outfile, "\t\tsds_code %d\n", ptr->sym_sds);
      fprintf(outfile, "\t}\n\n");

   }

}


/* ****************************************************************************
**************************************************************************** */
int symbol_lut_type::read_data(FILE *infile, char *token) {

   symbol_correlation_type *ptr;
   binary_node *ctr;
   bsdata t;

   dest();

   while (get_token(infile, token) && token[0] != '}') {

      insert(ptr = new symbol_correlation_type, NULL);

      ptr->sym_id = atoi(token);

      get_token(infile, token);  // '{'

      while (get_token(infile, token) && token[0] != '}') {

         if (!strcmp(token, "label")) {
            get_token(infile, token);
            ptr->sym_label.stringcpy(token);
            continue;
         }
	 
         if (!strcmp(token, "dis_code")) {
            get_token(infile, token);
            ptr->sym_sim.stringcpy(token);
            continue;
         }
	 
         if (!strcmp(token, "jpix_code")) {
            get_token(infile, token);
            ptr->sym_3D = atoi(token);
            continue;
         }
	 
         if (!strcmp(token, "gev_code")) {
            get_token(infile, token);
            ptr->sym_2D = atoi(token);
            continue;
         }
	 
         if (!strcmp(token, "gev_color")) {
            get_token(infile, token);
            t.key.stringcpy(token);
            ctr = colortree.find(&t);
	          
            if (ctr) {
               copyarray3(ptr->sym_2D_color, ((bsdata *)ctr->key)->color);
               ptr->sym_2D_colorname.stringcpy(token);
            }

            else {
               ptr->sym_2D_color[0] = ptr->sym_2D_color[1] = ptr->sym_2D_color[2] = 255;
               ptr->sym_2D_colorname.stringcpy("white");
            }
		  
            continue;
         }
	 
         if (!strcmp(token, "sds_code")) {
            get_token(infile, token);
            ptr->sym_sds = atoi(token);
            continue;
         }
	 
         if (!strcmp(token, "burst_code")) {
            get_token(infile, token);
            ptr->burst_warhead = atoi(token);

            get_token(infile, token);
            ptr->burst_fuze = atoi(token);

            get_token(infile, token);
            ptr->burst_quantity = atoi(token);

            get_token(infile, token);
            ptr->burst_rate = atoi(token);
            continue;
         }
	 
         sprintf(perror_buffer, "Warning:  Unknown setup token \"%s\"\n", token);
         pprintf(perror_buffer);
      }

   }

   return 1;
}


/* ****************************************************************************
**************************************************************************** */
void register_setup_CB(void *setupCB) { setup_CB = setupCB; }


/* ****************************************************************************
**************************************************************************** */
void register_setup_out_CB(void *setup_outCB) { setup_out_CB = setup_outCB; }


/* ****************************************************************************
**************************************************************************** */
int init_color_lut(char *filename) {

   FILE *infile;
   char token[MAXSTRLEN];
   bsdata *ptr;
   binary_node *bnode;
   int i;
   
   colortree.dest();
   
   if (!find_file(filename, "r", NULL, PLATFORM_SLASH, NULL, &infile)) {
      printf("Warning: Cannot access \"%s\"...\n", filename);
      return 0;
   }
   
   while (!feof(infile)) {
      get_token(infile, token);
      if (token[0] == '!') {
         fgets(token, MAXSTRLEN-1, infile);
         continue;
      }

      ptr = new bsdata;            
      ptr->color[0] = atoi(token);
      get_token(infile, token);
      ptr->color[1] = atoi(token);
      get_token(infile, token);
      ptr->color[2] = atoi(token);

      do {
         token[0] = fgetc(infile);
      } while (token[0] == ' ' || token[0] == '\t' || token[0] == '\n');

      fgets(&token[1], MAXSTRLEN-1, infile);

      for (i=0; token[i]; i++)
         if (token[i] == '/' && token[i+1] == '/') {
	    token[i] = 0;
	    break;
	 }

      for (i=strlen(token)-1; i > -1; i--)
         if (token[i] == ' ' || token[i] == '\t' || token[i] == '\n')
            token[i] = 0;
         else
            break;

      ptr->key.stringcpy(token);
      bnode = new binary_node;
      bnode->key = ptr;
      colortree.insert(bnode);
   }
  
   fclose(infile); 
   return 1;
}


/* **************************************************
************************************************** */
int parse_setup(char *filename) {

   FILE *infile;
   char token[MAXSTRLEN];
   
   terrain_name.stringcpy("terrain.atm");
   overlay_name.stringcpy("");
   map_indexname.stringcpy("");

   // parse setup file
   if (!find_file(filename, "r", NULL, PLATFORM_SLASH, NULL, &infile)) {
      sprintf(perror_buffer, "Warning: Unable to locate setup file \"%s\"...\n", filename);
      pprintf(perror_buffer);
      return 0;
   }

   while (get_token(infile, token)) {
      lower_case(token);

      switch (token[0]) {

         case '3':   // 3D_VIEW_DISTANCE
            if (get_token(infile, token))
               backplane = -atof(token);
            continue;

         case 'c':
            if (!strcmp(token, "classification")) {
               if (get_token(infile, token)) {
                  switch (token[0]) {
                     case 'u':
                        security_level = LEVEL_UNCLASS;
                        break;
			
                     case 's':
                        security_level = LEVEL_SECRET;
                        break;

                     default:
                        security_level = LEVEL_TOP_SECRET;
                        break;
                  }

               }

               continue;
            }

            if (!strcmp(token, "camera_distance")) {
               if (get_token(infile, token)) {
                  camera_dist = atof(token);
                  if (camera_dist < 1)
                     camera_dist = 1;
               }
	       
               continue;
            }

            if (!strcmp(token, "camera_height")) {
               if (get_token(infile, token))
                  camera_height = atof(token);
               continue;
            }

            if (!strcmp(token, "camera_azimuth")) {
               if (get_token(infile, token)) {
                  camera_azimuth = atof(token);
                  camera_azimuth = deg2rad(camera_azimuth);
               }
	       
               continue;
            }

            if (!strcmp(token, "camera_angular_velocity")) {
               if (get_token(infile, token)) {
                  camera_angular_velocity = atof(token);
                  camera_angular_velocity = deg2rad(camera_angular_velocity);
               }
	       
               continue;
            }

            break;

         case 'd':	// DEVELOPMENT_CORRELATION

            if (!strcmp("development_correlation", token)) {
               get_token(infile, token);	// "{"
               symbol_lut.read_data(infile, token);
               continue;
            }

            break;

         case 'f':
            if (!strcmp(token, "fog")) {
               if (get_token(infile, token)) {
                  lower_case(token);

                  if (!strcmp(token, "foreground"))
                     local_game_fogflag[FOREGROUND] = 1;
                  else if (!strcmp(token, "midground"))
                     local_game_fogflag[MIDGROUND] = 1;
                  else if (!strcmp(token, "background"))
                     local_game_fogflag[BACKGROUND] = 1;
               }
	       
               continue;
            }
	 
            if (!strcmp(token, "fog_start")) {

               if (get_token(infile, token))
                  local_game_fog_start = atof(token);

               continue;
            }
	 
            if (!strcmp(token, "fog_end")) {

               if (get_token(infile, token))
                  local_game_fog_end = atof(token);

               continue;
            }
	 
            if (!strcmp(token, "fog_color")) {

               if (get_token(infile, token))
                  local_game_fog_color[0] = atoi(token);
               if (get_token(infile, token))
                  local_game_fog_color[1] = atoi(token);
               if (get_token(infile, token))
                  local_game_fog_color[2] = atoi(token);

               continue;
            }
	    
            break;

         case 'i': // tile base_name count
            get_token(infile, token);
            map_indexname.stringcpy(token);
            continue;

         case 'j':
            controller = CONTROLJOYSTICK;
            get_token(infile, token);
            controller_speed[CONTROLJOYSTICK] = atof(token);
            continue;
	    
         case 'k':
            controller = CONTROLKEYBOARD;
            get_token(infile, token);
            controller_speed[CONTROLKEYBOARD] = atof(token);
            continue;

         case 'm':
            controller = CONTROLMOUSE;
            get_token(infile, token);
            controller_speed[CONTROLMOUSE] = atof(token);
            continue;
	 
         case 'o':
            if (!strcmp(token, "opengl")) {
               get_token(infile, token);
               openglflag = atoi(token);
               continue;
            }

            if (!strcmp(token, "overlay")) {
               get_token(infile, token);
               overlay_name.stringcpy(token);
               continue;
            }

            if (!strcmp(token, "overlay_intensity")) {
               get_token(infile, token);
               overlay_intensity = (int)(atoi(token)*2.55);
               continue;
            }

            break;
	 
         case 'p':	// PIXCON_PATH
            sprintf(token, "PIXCON_PATH");
            if (get_token(infile, token)) {
               PIXCON_PATH.stringcpy("PIXCON_PATH=");
               PIXCON_PATH.stringcat(token);
               putenv(PIXCON_PATH.string);
            }

            continue;

         case 't':	// TERRAIN_FILE
            if (!strcmp(token, "terrain_file")) {
               get_token(infile, token);
               terrain_name.stringcpy(token);
               continue;
            }

            if (!strcmp(token, "tile_color")) {
               get_token(infile, token);
               tile_color[0] = atoi(token);

               get_token(infile, token);
               tile_color[1] = atoi(token);

               get_token(infile, token);
               tile_color[2] = atoi(token);

               get_token(infile, token);
               tile_color[3] = atoi(token);
               continue;
            }

            break;

         default:
            break;
      }

      ((void (*)(FILE *, char *))setup_CB)(infile, token);
   }

   fclose(infile);
   return 1;
}


/* **************************************************
************************************************** */
int write_setup(char *filename) {

   FILE *outfile;
   char *ctr;

   // parse setup file
   outfile = fopen(filename, "w");

   if (!outfile)
      return 0;
   
   fprintf(outfile, "3d_view_distance %f\n", -backplane);

   switch (security_level) {
   
      case LEVEL_UNCLASS:
         fprintf(outfile, "classification unclassified\n");
         break;
 
      case LEVEL_SECRET:
         fprintf(outfile, "classification secret\n");
         break;
 
      case LEVEL_TOP_SECRET:
      default:
         fprintf(outfile, "classification top_secret\n");
         break;
   } 

   fprintf(outfile, "camera_distance %f\n", camera_dist);
   fprintf(outfile, "camera_height %f\n",   camera_height);
   fprintf(outfile, "camera_azimuth %f\n",  rad2deg(camera_azimuth));
   fprintf(outfile, "camera_camera_angular_velocity %f\n", rad2deg(camera_angular_velocity));

   if (symbol_lut.head) {
      fprintf(outfile, "development_correlation {\n");
      symbol_lut.dump(outfile);
      fprintf(outfile, "}\n");
   }
   
   if (local_game_fogflag[FOREGROUND])
      fprintf(outfile, "fog foreground\n");

   if (local_game_fogflag[MIDGROUND])
      fprintf(outfile, "fog midground\n");

   if (local_game_fogflag[BACKGROUND])
      fprintf(outfile, "fog background\n");

   fprintf(outfile, "fog_start %f\n", local_game_fog_start);
   fprintf(outfile, "fog_end %f\n", local_game_fog_end);
   fprintf(outfile, "fog_color %d %d %d\n", local_game_fog_color[0], local_game_fog_color[1], local_game_fog_color[2]);
   fprintf(outfile, "image %s\n", map_indexname.string);

   switch (controller) {
      case CONTROLJOYSTICK:
         fprintf(outfile, "joystick %f\n", controller_speed[CONTROLJOYSTICK]);
         break;
	 
      case CONTROLKEYBOARD:
         fprintf(outfile, "keyboard %f\n", controller_speed[CONTROLKEYBOARD]);
         break;
	 
//      case CONTROLMOUSE:
      default:
         fprintf(outfile, "mouse %f\n", controller_speed[CONTROLMOUSE]);
         break;
   }
   
   fprintf(outfile, "opengl %d\n", openglflag);

   if (overlay_name.string)
      fprintf(outfile, "overlay %s\n", overlay_name.string);

   fprintf(outfile, "overlay_intensity %d\n", (int)(overlay_intensity/2.55));

   ctr = getenv("PIXCON_PATH");
   
   if (ctr)
      fprintf(outfile, "PIXCON_PATH %s\n", ctr);

   if (scramnet_flag)
      fprintf(outfile, "scramnet\n");
   else
      fprintf(outfile, "shmem\n");

   if (terrain_name.string)
      fprintf(outfile, "terrain_file %s\n", terrain_name.string);
   
   fprintf(outfile, "tile_color %d %d %d\n", tile_color[0], tile_color[1], tile_color[2]);

   ((void (*)(FILE *))setup_out_CB)(outfile);

   fclose(outfile);
   return 1;
}
